@@ -1,7 +1,8 @@
 #' @importFrom dplyr show_query
 #' @export
-show_query.tbl_lazy <- function(x, ...) {
-  sql <- remote_query(x)
+show_query.tbl_lazy <- function(x, ..., cte = FALSE) {
+  withr::local_options(list(dbplyr_use_colour = TRUE))
+  sql <- remote_query(x, cte = cte)
   cat_line("<SQL>")
   cat_line(sql)
   invisible(x)
@@ -14,7 +15,7 @@ explain.tbl_sql <- function(x, ...) {
   show_query(x)
   cat_line()
   cat_line("<PLAN>")
-  cat_line(remote_query_plan(x))
+  cat_line(remote_query_plan(x, ...))
 
   invisible(x)
 }
