@@ -99,9 +99,8 @@ rows_insert.tbl_lazy <- function(x,
 
   conflict <- rows_check_conflict(conflict)
 
-  y <- auto_copy(x, y, copy = copy)
-
   rows_check_containment(x, y)
+  y <- rows_auto_copy(x, y, copy = copy)
 
   by <- rows_check_by(by, y)
 
@@ -156,9 +155,8 @@ rows_append.tbl_lazy <- function(x,
   rows_check_in_place(x, in_place)
   name <- target_table_name(x, in_place)
 
-  y <- auto_copy(x, y, copy = copy)
-
   rows_check_containment(x, y)
+  y <- rows_auto_copy(x, y, copy = copy)
 
   returning_cols <- rows_check_returning(x, returning, enexpr(returning))
 
@@ -204,9 +202,8 @@ rows_update.tbl_lazy <- function(x,
   rows_check_in_place(x, in_place)
   name <- target_table_name(x, in_place)
 
-  y <- auto_copy(x, y, copy = copy)
-
   rows_check_containment(x, y)
+  y <- rows_auto_copy(x, y, copy = copy)
 
   by <- rows_check_by(by, y)
 
@@ -285,9 +282,8 @@ rows_patch.tbl_lazy <- function(x,
   rows_check_in_place(x, in_place)
   name <- target_table_name(x, in_place)
 
-  y <- auto_copy(x, y, copy = copy)
-
   rows_check_containment(x, y)
+  y <- rows_auto_copy(x, y, copy = copy)
 
   by <- rows_check_by(by, y)
 
@@ -307,6 +303,7 @@ rows_patch.tbl_lazy <- function(x,
     }
 
     con <- remote_con(x)
+
     update_cols <- setdiff(colnames(y), by)
     update_values <- sql_coalesce(
       sql_table_prefix(con, update_cols, name),
@@ -374,9 +371,8 @@ rows_upsert.tbl_lazy <- function(x,
   rows_check_in_place(x, in_place)
   name <- target_table_name(x, in_place)
 
-  y <- auto_copy(x, y, copy = copy)
-
   rows_check_containment(x, y)
+  y <- rows_auto_copy(x, y, copy = copy)
 
   by <- rows_check_by(by, y)
 
@@ -450,9 +446,8 @@ rows_delete.tbl_lazy <- function(x,
   rows_check_in_place(x, in_place)
   name <- target_table_name(x, in_place)
 
-  y <- auto_copy(x, y, copy = copy)
-
   rows_check_containment(x, y)
+  y <- rows_auto_copy(x, y, copy = copy)
 
   by <- rows_check_by(by, y)
 
@@ -645,15 +640,13 @@ rows_check_key <- function(x,
   }
 }
 
-rows_check_in_place <- function(df, in_place) {
-  if (!rlang::is_bool(in_place)) {
-    cli_abort("{.arg in_place} must be `TRUE` or `FALSE`.")
-  }
+rows_check_in_place <- function(df, in_place, call = caller_env()) {
+  check_bool(in_place, call = call)
 
   if (!in_place) return()
 
   if (inherits(df, "tbl_TestConnection")) {
-    cli_abort("{.code in_place = TRUE} does not work for simulated connections.")
+    cli_abort("{.code in_place = TRUE} does not work for simulated connections.", call = call)
   }
 }
 
@@ -665,16 +658,7 @@ rows_check_conflict <- function(conflict, error_call = caller_env()) {
     error_call = error_call
   )
 
-  if (conflict == "error") {
-    cli_abort(
-      c(
-        '{.code conflict = "error"} is not supported for database tables.',
-        i = 'Please use {.code conflict = "ignore"} instead'
-      ),
-      call = error_call
-    )
-  }
-
+  check_unsupported_arg(conflict, "ignore", call = error_call)
   conflict
 }
 
@@ -686,15 +670,14 @@ rows_check_ummatched <- function(unmatched, error_call = caller_env()) {
     error_call = error_call
   )
 
-  if (unmatched == "error") {
-    cli_abort('{.code unmatched = "error"} is not supported for database tables.', call = error_call)
-  }
+  check_unsupported_arg(unmatched, "ignore", call = error_call)
 
   unmatched
 }
 
 rows_check_returning <- function(df, returning, returning_expr, error_call = caller_env()) {
-  returning_cols <- eval_select2(returning_expr, df)
+  locs <- tidyselect::eval_select(returning_expr, df, error_call = error_call)
+  returning_cols <- set_names(colnames(df)[locs], names(locs))
 
   if (is_empty(returning_cols)) return(returning_cols)
 
@@ -724,13 +707,6 @@ tick <- function(x) {
 
 # other helpers -----------------------------------------------------------
 
-eval_select2 <- function(expr, data) {
-  sim_data <- simulate_vars(data)
-  locs <- tidyselect::eval_select(expr, sim_data)
-  names_out <- names(locs)
-  set_names(colnames(sim_data)[locs], names_out)
-}
-
 target_table_name <- function(x, in_place) {
   # Never touch target table with `in_place = FALSE`
   if (!is_true(in_place)) {
@@ -753,7 +729,7 @@ rows_prep <- function(con, x_name, y, by, lvl = 0) {
     lvl = lvl
   )
 
-  join_by <- list(x = by, y = by, x_as = y_name, y_as = x_name)
+  join_by <- list(x = by, y = by, x_as = y_name, y_as = x_name, condition = "=")
   where <- sql_join_tbls(con, by = join_by, na_matches = "never")
 
   list(
@@ -765,7 +741,7 @@ rows_prep <- function(con, x_name, y, by, lvl = 0) {
 rows_insert_prep <- function(con, x_name, y, by, lvl = 0) {
   out <- rows_prep(con, x_name, y, by, lvl = lvl)
 
-  join_by <- list(x = by, y = by, x_as = x_name, y_as = ident("...y"))
+  join_by <- list(x = by, y = by, x_as = x_name, y_as = ident("...y"), condition = "=")
   where <- sql_join_tbls(con, by = join_by, na_matches = "never")
   out$conflict_clauses <- sql_clause_where_exists(x_name, where, not = TRUE)
 
@@ -775,14 +751,63 @@ rows_insert_prep <- function(con, x_name, y, by, lvl = 0) {
   out
 }
 
-rows_get_or_execute <- function(x, sql, returning_cols) {
-  con <- remote_con(x)
-  if (is_empty(returning_cols)) {
-    dbExecute(con, sql, immediate = TRUE)
-  } else {
-    returned_rows <- dbGetQuery(con, sql, immediate = TRUE)
-    x <- set_returned_rows(x, returned_rows)
+rows_auto_copy <- function(x, y, copy, call = caller_env()) {
+  name <- remote_name(x)
+  x_types <- get_col_types(remote_con(x), name, call)
+
+  if (!is_null(x_types)) {
+    rows_check_containment(x, y, error_call = call)
+    x_types <- x_types[colnames(y)]
   }
 
+  auto_copy(x, y, copy = copy, types = x_types)
+}
+
+get_col_types <- function(con, name, call) {
+  if (is_null(name)) {
+    return(NULL)
+  }
+
+  UseMethod("get_col_types")
+}
+
+#' @export
+get_col_types.TestConnection <- function(con, name, call) {
+  NULL
+}
+
+#' @export
+get_col_types.DBIConnection <- function(con, name, call) {
+  NULL
+}
+
+#' @export
+get_col_types.PqConnection <- function(con, name, call) {
+  name <- as.sql(name, con)
+  res <- DBI::dbSendQuery(con, paste0("SELECT * FROM ", name))
+  on.exit(DBI::dbClearResult(res))
+  DBI::dbFetch(res, n = 0)
+  col_info_df <- DBI::dbColumnInfo(res)
+  set_names(col_info_df[[".typname"]], col_info_df[["name"]])
+}
+
+rows_get_or_execute <- function(x, sql, returning_cols, call = caller_env()) {
+  con <- remote_con(x)
+  msg <- "Can't modify database table {.val {remote_name(x)}}."
+  tryCatch(
+    {
+      if (is_empty(returning_cols)) {
+        DBI::dbExecute(con, sql, immediate = TRUE)
+      } else {
+        returned_rows <- DBI::dbGetQuery(con, sql, immediate = TRUE)
+        x <- set_returned_rows(x, returned_rows)
+      }
+    },
+    error = function(cnd) {
+      cli_abort(msg, parent = cnd, call = call)
+    }
+  )
+
+
   invisible(x)
 }
