@@ -47,7 +47,9 @@ sql_call2 <- function(.fn, ..., con = sql_current_con()) {
 
 
 replace_expr <- function(x, con) {
-  if (is.atomic(x) || blob::is_blob(x)) {
+  if (is.null(x)) {
+    "NULL"
+  } else if (is.atomic(x) || blob::is_blob(x)) {
     as.character(escape(unname(x), con = con))
   } else if (is.name(x)) {
     as.character(x)
