@@ -99,16 +99,87 @@ local_methods <- function(..., .frame = caller_env()) {
   local_bindings(..., .env = global_env(), .frame = .frame)
 }
 
-assert_flag <- function(x, arg, call = caller_env()) {
-  vctrs::vec_assert(x, logical(), size = 1L)
-  if (is.na(x)) {
-    cli_abort("{.arg {arg}} must not be NA.", call = call)
+check_list <- function(x, ..., allow_null = FALSE, arg = caller_arg(x), call = caller_env()) {
+  if (vctrs::vec_is_list(x)) {
+    return()
+  }
+  stop_input_type(
+    x,
+    c("a list"),
+    ...,
+    allow_na = FALSE,
+    allow_null = allow_null,
+    arg = arg,
+    call = call
+  )
+}
+
+check_lazy_query <- function(x, ..., arg = caller_arg(x), call = caller_env()) {
+  if (!inherits(x, "lazy_query")) {
+    stop_input_type(
+      x,
+      what = "a lazy query",
+      ...,
+      arg = arg,
+      call = call
+    )
   }
 }
 
-check_not_supplied <- function(arg, call = caller_env()) {
-  if (!is_null(arg)) {
-    arg <- caller_arg(arg)
-    cli_abort("{.arg {arg}} is not supported in SQL translations.", call = call)
+check_con <- function(con, ..., arg = caller_arg(con), call = caller_env()) {
+  if (is.null(con)) {
+    cli_abort("{.arg {arg}} must not be NULL.", call = call)
+  }
+}
+
+check_unsupported_arg <- function(x,
+                                  allowed = NULL,
+                                  ...,
+                                  backend = NULL,
+                                  arg = caller_arg(x),
+                                  call = caller_env()) {
+  if (is_missing(x)) {
+    return()
+  }
+
+  if (identical(x, allowed)) {
+    return()
+  }
+
+  if (is_null(allowed)) {
+    msg <- "Argument {.arg {arg}} isn't supported"
+  } else {
+    msg <- "{.code {arg} = {.val {x}}} isn't supported"
+  }
+
+  if (is.null(backend)) {
+    msg <- paste0(msg, " on database backends.")
+  } else {
+    msg <- paste0(msg, " in {backend} translation.")
+  }
+
+  if (!is_null(allowed)) {
+    msg <- c(
+      msg,
+      i = "It must be {.val {allowed}} instead."
+    )
+  }
+  cli_abort(msg, call = call)
+}
+
+stop_unsupported_function <- function(f, ..., with = NULL, call = caller_env()) {
+  cli_abort(c(
+    "{.fun {f}} is not supported on database backends.",
+    i = if (!is_null(with)) "Please use {.fun {with}} instead."
+  ), call = call)
+}
+
+check_named <- function(x, ..., arg = caller_arg(x), call = caller_env()) {
+  if (!is_named2(x)) {
+    cli_abort("All elements of {.arg {arg}} must be named.", call = call)
+  }
+
+  if (vctrs::vec_duplicate_any(names2(x))) {
+    cli_abort("The names of {.arg {arg}} must be unique.", call = call)
   }
 }
