@@ -163,6 +163,11 @@ test_that("LHS can handle different types", {
     sql("CASE WHEN (`z` IN (1)) THEN 'a' END")
   )
 
+  expect_equal(
+    translate_sql(case_match(z, "x" ~ "a")),
+    sql("CASE WHEN (`z` IN ('x')) THEN 'a' END")
+  )
+
   expect_equal(
     translate_sql(case_match(z, y ~ "a")),
     sql("CASE WHEN (`z` IN (`y`)) THEN 'a' END")
@@ -185,6 +190,11 @@ test_that("LHS can match multiple values", {
     sql("CASE WHEN (`z` IN (1, 3)) THEN 'a' END")
   )
 
+  expect_equal(
+    translate_sql(case_match(z, c("x", "y") ~ "a")),
+    sql("CASE WHEN (`z` IN ('x', 'y')) THEN 'a' END")
+  )
+
   expect_equal(
     translate_sql(case_match(z, c(1L, y) ~ "a")),
     sql("CASE WHEN (`z` IN (1, `y`)) THEN 'a' END")
